package compact

import (
	"bufio"
	"fmt"
	"io"
	"math"
	"sort"
	"strings"

	"github.com/fatih/color"
	. "ser1.net/legume"
)

// TODO add context, project to output
// TODO Add non-color priority for external filtering tools
func ToString(is []Issue, out io.Writer) {
	sort.Slice(is, func(i, j int) bool {
		if is[i].Priority == is[j].Priority {
			if is[i].Type == is[j].Type {
				return is[i].Row < is[j].Row
			}
			return is[i].Type < is[j].Type
		}
		if is[i].Priority == "" {
			return false
		}
		if is[j].Priority == "" || (is[i].Priority < is[j].Priority) {
			return true
		}
		return false
	})
	var maxdw int
	var maxfw int
	for _, i := range is {
		digits := int(math.Log10(float64(i.Row))) + 1
		if digits > maxdw {
			maxdw = digits
		}
		if len(i.Path) > maxfw {
			maxfw = len(i.Path)
		}
	}
	//fstr := fmt.Sprintf("%%%ds:%%-%dd %%s ", maxfw, maxdw)
	tagKey := color.FgHiBlue
	tagVal := color.FgHiMagenta
	priorities := map[string]color.Attribute{
		"A": color.FgRed,
		"B": color.FgHiRed,
		"C": color.FgYellow,
		"D": color.FgHiYellow,
		"E": color.FgGreen,
		"F": color.FgHiGreen,
	}
	priElse := color.FgWhite
	fnum := fmt.Sprintf("%%%dd", maxdw)
	paddingC := maxdw + 12
	paddingA := maxdw + 5
	var state string
	for c, i := range is {
		base := color.New(color.Reset)
		switch i.Change {
		case NEW:
			base = color.New(color.Bold)
			state = base.Add(color.FgYellow).Sprintf(" %6s ", Changes[i.Change])
		case CLOSED:
			base = color.New(color.CrossedOut)
			state = base.Add(color.FgGreen).Sprintf(" %6s ", Changes[i.Change])
		default:
			state = " "
		}
		priCol, found := priorities[i.Priority]
		if !found {
			priCol = priElse
		}
		num := base.Add(priCol).Sprintf(fnum, c+1)
		base.Fprintf(out, "%s %s%s%s ", num, typ(base, i), state, padDesc(i, paddingA, paddingC))
		l := len(i.Tags)
		idx := 0
		if l > 0 {
			base.Fprint(out, "[")
			idx++
			for k, v := range i.Tags {
				base.Add(tagKey).Fprintf(out, k)
				base.Fprintf(out, ":")
				base.Add(tagVal).Fprintf(out, v)
				if idx < l {
					base.Fprint(out, " ")
				}
			}
			base.Fprint(out, "]")
		}
		fmt.Fprintln(out)
	}
}

// Pads the lines after the first with n spaces
func padDesc(i Issue, a, c int) string {
	pad := a
	if i.Change > 0 {
		pad = c
	}
	padS := fmt.Sprintf("\n%%%ds%%s", pad)
	var rv string
	s := bufio.NewScanner(strings.NewReader(i.Description))
	for s.Scan() {
		if len(rv) == 0 {
			rv += s.Text()
		} else {
			rv += fmt.Sprintf(padS, "", s.Text())
		}
	}
	return rv
}

func typ(b *color.Color, i Issue) string {
	switch i.Type {
	case "TODO":
		return b.Add(color.FgGreen).SprintFunc()("REQ")
	case "FIXME":
		return b.Add(color.FgRed).SprintFunc()("BUG")
	case "XXX":
		return b.Add(color.FgYellow).SprintFunc()("INF")
	default:
		return b.Add(color.FgMagenta).SprintFunc()(i.Type)
	}
}
