package detail

import (
	"fmt"
	"io"
	"math"
	"sort"
	"strings"

	"github.com/fatih/color"
	. "ser1.net/legume"
)

func ToString(is []Issue, out io.Writer) {
	sort.Slice(is, func(i, j int) bool {
		path := strings.Compare(is[i].Path, is[j].Path)
		if path == 0 {
			if is[i].Priority == is[j].Priority {
				if is[i].Type == is[j].Type {
					return is[i].Row < is[j].Row
				}
				return is[i].Type < is[j].Type
			}
			if is[i].Priority == "" {
				return false
			}
			if is[j].Priority == "" || (is[i].Priority < is[j].Priority) {
				return true
			}
		}
		return path < 0
	})
	var maxw int
	for _, i := range is {
		digits := int(math.Log10(float64(i.Row))) + 1
		if digits > maxw {
			maxw = digits
		}
	}
	fstr := fmt.Sprintf("%%s %%%dd %%s ", maxw)
	var dir string
	var cr string // Skip the blank line before the first file
	clr := color.New(color.Underline)
	tagKey := color.New(color.FgHiBlue)
	tagVal := color.New(color.FgHiMagenta)
	for _, i := range is {
		if dir != i.Path {
			dir = i.Path
			clr.Fprintf(out, "%s%s\n", cr, dir)
			cr = "\n"
		}
		fmt.Fprintf(out, fstr, typ(i), i.Row+1, i.Description)
		l := len(i.Tags)
		idx := 0
		if l > 0 {
			fmt.Fprint(out, "[")
			idx++
			for k, v := range i.Tags {
				tagKey.Fprint(out, k)
				fmt.Fprint(out, ":")
				tagVal.Fprint(out, v)
				if idx < l {
					fmt.Fprint(out, " ")
				}
			}
			fmt.Fprint(out, "]")
		}
		fmt.Fprintln(out)
	}
}

func typ(i Issue) string {
	switch i.Type {
	case "TODO":
		return color.New(color.FgGreen).SprintFunc()("REQ")
	case "FIXME":
		return color.New(color.FgRed).SprintFunc()("BUG")
	case "XXX":
		return color.New(color.FgYellow).SprintFunc()("INF")
	default:
		return color.New(color.FgMagenta).SprintFunc()(i.Type)
	}
}
