package details

import (
	"fmt"
	"io"
	"sort"
	"strings"

	"github.com/fatih/color"
	wrap "github.com/mitchellh/go-wordwrap"
	. "ser1.net/legume"
)

func ToString(idxs []int, is []Issue, out io.Writer) {
	for _, idx := range idxs {
		if idx > len(is) || idx < 1 {
			fmt.Fprintf(out, "invalid issue number %d; must be 1 to %d\n", idx, len(is))
			return
		}
		sort.Slice(is, func(i, j int) bool {
			if is[i].Priority == is[j].Priority {
				if is[i].Type == is[j].Type {
					return is[i].Row < is[j].Row
				}
				return is[i].Type < is[j].Type
			}
			if is[i].Priority == "" {
				return false
			}
			if is[j].Priority == "" || (is[i].Priority < is[j].Priority) {
				return true
			}
			return false
		})
		i := is[idx-1]

		color.New(color.Underline).Fprintf(out, i.Path)
		fmt.Fprint(out, ": ")
		color.New(color.FgGreen).Add(color.Underline).Fprintf(out, "%d\n", i.Row)

		dl := color.New(color.FgCyan)
		tagKey := color.New(color.FgHiBlue)
		tagVal := color.New(color.FgHiMagenta)
		priorities := map[string]color.Attribute{
			"A": color.FgRed,
			"B": color.FgHiRed,
			"C": color.FgYellow,
			"D": color.FgHiYellow,
			"E": color.FgGreen,
			"F": color.FgHiGreen,
		}
		dl.Fprint(out, "Type:")
		fmt.Fprintf(out, " %s\n", typ(i))
		if len(i.Priority) > 0 {
			pri, ok := priorities[i.Priority]
			if !ok {
				pri = color.FgWhite
			}
			dl.Fprint(out, "Priority:")
			color.New(pri).Fprintf(out, " %s\n", i.Priority)
		}
		if len(i.Contexts) > 0 {
			dl.Fprint(out, "Contexts:")
			fmt.Fprintf(out, " %s\n", strings.Join(i.Contexts, ", "))
		}
		if len(i.Projects) > 0 {
			dl.Fprint(out, "Projects:")
			fmt.Fprintf(out, " %s\n", strings.Join(i.Projects, ", "))
		}
		if i.Date != nil {
			dl.Fprint(out, "Created:")
			fmt.Fprintf(out, " %s\n", i.Date.Format("2006-01-02"))
		}

		l := len(i.Tags)
		if l > 0 {
			dl.Fprint(out, "Tags:")
			fmt.Fprint(out, " ")
			tidx := 0
			if l > 0 {
				tidx++
				for k, v := range i.Tags {
					tagKey.Fprint(out, k)
					fmt.Fprint(out, ":")
					tagVal.Fprint(out, v)
					if tidx < l {
						fmt.Fprint(out, ", ")
					}
				}
			}
			fmt.Fprintln(out)
		}

		dl.Fprintln(out, "Description: ")
		fmt.Fprintf(out, "  %s\n", wrap.WrapString(i.Description, 75))
	}
}

func typ(i Issue) string {
	switch i.Type {
	case "TODO":
		return color.New(color.FgGreen).SprintFunc()("REQ")
	case "FIXME":
		return color.New(color.FgRed).SprintFunc()("BUG")
	case "XXX":
		return color.New(color.FgYellow).SprintFunc()("INF")
	default:
		return color.New(color.FgMagenta).SprintFunc()(i.Type)
	}
}
