// Package legume is a distributed issue tracker
//
// Inspired by lentil and following the philosophy of todo.sh, uses comment
// metadata in code to track issues. Lentil is an inspired concept and a nice
// tool; legume should be considered flattery, not criticism.
//
// Legume leverages the developer tradition of using code comments to make
// notes.  Notes of this sort retain locality; they are reminders for code
// browsers, and contain meta-data about where to start looking when addressing
// an issue. They integrate naturally with whatever VCS the developer is using,
// and make it easy for a developer to remember to remove the comment when the
// topic is addressed. It requires no extra external database.
//
// The trade-offs, which may or may not be considered limitations, include:
// * No unique IDs, which can make referencing tickets difficult or cumbersome
// * Very simple tickets.  No attachments, no cross-references or dependencies,
//   no robust commenting or history tracking.
// * Efficiency. In a pure state, collating tickets requires walking directory
//   trees and parsing source files. This is addressed by caching, but caches
//   introduce opportunities for sync errors; it's a necessary wart on an
//   otherwise elegant and simple solution.
//
// Goals beyond lentil
// * Accept diffs & STDIN This + diffs allows some ability to track history by
//   using the output of a VCS.  E.g., hg diff -r <histnode> | glentil would
//   produce a list of all issues added and resolved, using diff markup (+/-) to
//   indicate opened/closed
// * Re-implement in something lower-impact to compile than Haskell; the core
//   Haskell stack (incl. common libraries) is 183 packages in Arch and 1.9GB.
// * Maintain Haskell's ability to generate distributable statically linked
//   binaries
// * Improve performance; 144 files & 19k LOC takes lentil 1.1s to process
//
// Project with 221 files, 32,579 lines:
//   ➜  time lentil -a go%c > /dev/null
//   lentil: out of memory (requested 1048576 bytes)
//   lentil . -a go%cpp -x vendor -a go%c  33.87s user 1.85s system 99% cpu 35.783 total
//   ➜  time leg > /dev/null
//   leg .  2.10s user 0.13s
//   system 111% cpu 2.012 total
//
// Project with 279 files, 16,156 lines:
//   ➜  time lentil -a go%c . > /tmp/lentil.out
//   lentil -a go%c .  19.54s user 0.28s
//   system 99% cpu 19.846 total
//   ➜  time leg . > /tmp/leg.out
//   leg .   0.82s user 0.03s
//   system 104% cpu 0.815 total

package legume
