package fs

import (
	"path/filepath"
	"testing"

	. "ser1.net/legume"
)

func Test_makeSourceDir(t *testing.T) {
	type args struct {
		a   map[string]string
		f   func(s string) bool
		dir string
	}
	tests := []struct {
		name string
		args args
		wsrc int
		werr int
	}{
		{"no dir", args{map[string]string{}, func(s string) bool { return true }, "blahblah"}, 0, 1},
		{"test", args{map[string]string{}, func(s string) bool { return true }, "../test"}, 10, 0},
		{"func no", args{map[string]string{}, func(s string) bool { return false }, "../test"}, 0, 0},
		// 7, because we ignore suffixes we don't handle, e.g. .diff
		{"func no c", args{map[string]string{}, func(s string) bool { return filepath.Ext(s) != ".c" }, "../test"}, 7, 0},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			cfs := make(chan SourceFile)
			cer := make(chan error)
			go func() {
				makeSourceDir(tt.args.a, tt.args.f, []string{tt.args.dir}, cfs, cer)
				close(cfs)
				close(cer)
			}()
			var nfs, ner int
			eok := true
			fok := true
			for eok || fok {
				select {
				case _, eok = <-cer:
					if eok {
						ner++
					}
				case _, fok = <-cfs:
					if fok {
						nfs++
					}
				}
			}
			if nfs != tt.wsrc {
				t.Errorf("Test_makeSourceDir() %s nfs = %v, want %v", tt.name, nfs, tt.wsrc)
			}
			if ner != tt.werr {
				t.Errorf("Test_makeSourceDir() %s ner = %v, want %v", tt.name, ner, tt.werr)
			}
		})
	}
}
