package legume

import (
	"time"
)

const (
	// ACTIVE denotes a non-diff ticket
	ACTIVE = iota
	// NEW denotes a newly added ticket from a diff
	NEW
	// CLOSED denotes a ticket removed by a diff
	CLOSED
)

var Changes []string = []string{
	"ACTIVE",
	"NEW",
	"CLOSED",
}

// Issue contains all of the information parsed out of a comment.  The format
// of a sourcecode comment that creates an issue is, at minimum:
// 	// FIXME this is really broken
// Additional metadata can be added by using todo.txt structures, i.e.:
// * key:value pairs
// * @context keywords
// * +project keywords
// * A creation date in the format 2006-01-02
// * A priority in the format (A)
type Issue struct {
	Path        string
	Row         int
	Description string
	// key:value pairs
	Tags map[string]string
	// @context
	Contexts []string
	// +project
	Projects []string
	// Creation date, e.g. 2017-04-30
	Date *time.Time
	// (A) priority
	Priority string
	// Change type: ACTIVE, NEW, CLOSED (the latter two only for diffs)
	Change int
	// Type of issue: To-do (todo), Fix me (fixme), Note (xxx), or user-defined
	Type string
}
