declare-option -hidden int leg_current_todo_line

add-highlighter shared/leg group
add-highlighter shared/leg/ regex "^([^:\n]+):(\d+):(\d+) (?:(TODO)|(XXX)|(FIXME)|(BUG)).*?$" 1:cyan 2:green 3:green 4:green 5:cyan 6:yellow 7:red
add-highlighter shared/leg/ regex "^\h*(~*(?:(\^)~*)?)$" 1:green 2:cyan+b
add-highlighter shared/leg/ line '%opt{leg_current_todo_line}' default+b

define-command -docstring %{Navigate source code to-do comments} leg %{
    evaluate-commands %sh{
        output=$(mktemp -d "${TMPDIR:-/tmp}"/kak-leg.XXXXXXXX)/fifo
        mkfifo ${output}
        ( eval leg -f vim --output ${output} 2>&1 & ) > /dev/null 2>&1 < /dev/null
        printf %s\\n "evaluate-commands %{
            edit! -fifo ${output} *leg*
            set-option buffer filetype leg
            set-option buffer leg_current_todo_line 0
            hook -always -once buffer BufCloseFifo .* %{
                nop %sh{ rm -r $(dirname ${output}) }
            }
        }"
    }
}

hook -group leg-highlight global WinSetOption filetype=leg %{
    add-highlighter window/leg ref leg
    hook -once -always window WinSetOption filetype=.* %{ remove-highlighter window/leg }
}

hook global WinSetOption filetype=leg %{
    hook buffer -group make-hooks NormalKey <ret> leg-jump
    hook -once -always window WinSetOption filetype=.* %{ remove-hooks buffer leg-hooks }
}

define-command -hidden leg-open-todo -params 5 %{
    evaluate-commands %{
        edit -existing "%arg{1}" %arg{2} %arg{3}
        echo -markup "{StatusLine}%arg{4}{Default}"
        try %{ focus }
    }
}

# FIXME kakoune jump command jumps to wrong location, e.g kakoune/rc/tools/go/go-tools.kak#100 FIXME foobar
define-command -hidden leg-jump %{
    evaluate-commands %{
        execute-keys <a-h><a-l> s "^([^:\n]+):(\d+):(\d+) (TODO|XXX|FIXME|BUG) ([^\n]+)" <ret>l
        set-option buffer leg_current_todo_line %val{cursor_line}
        leg-open-todo "%reg{1}" "%reg{2}" "%reg{3}" "%reg{4}" "%reg{5}"
    }
}

define-command leg-next-todo -docstring 'Jump to the next leg todo' %{
    evaluate-commands %{
        buffer '*leg*'
        execute-keys "%opt{leg_current_todo_line}ggl" "/^[^:\n]+:\d+:\d+ (?:TODO|XXX|FIXME|BUG) [^\n]+<ret>"
        leg-jump
    }
}

define-command leg-previous-todo -docstring 'Jump to the previous leg todo' %{
    evaluate-commands %{
        buffer '*leg*'
        execute-keys "%opt{leg_current_todo_line}ggh" "<a-/>^[^:\n]+:\d+:\d+ (?:TODO|XXX|FIXME|BUG) [^\n]+<ret>"
        leg-jump
    }
}
