package parse

import (
	"fmt"
	"strings"

	. "ser1.net/legume"
)

func Parse(s SourceFile) []Issue {
	typ := Languages[s.Type()]
	comments := parse(s, typ)
	rv := make([]Issue, 0, len(comments))
	for _, cmt := range comments {
		for i, issue := range parseIssues(cmt) {
			issue.Row = cmt.Line + i
			issue.Path = s.Path()
			rv = append(rv, issue)
		}
	}
	return rv
}

type comment struct {
	String string
	Line   int
	Change int
}

func parse(i SourceFile, syn syntax) []comment {
	var line string
	var inComment, singleComment, inString, continueSingleLine bool
	var curCmt, end []rune
	// If we're in the middle of a comment start/end match, ridx is the char index into the array
	var ridx, cstart int
	comments := make([]comment, 0, 1000)

	addComment := func() {
		if singleComment && len(strings.TrimSpace(string(curCmt))) == 0 {
			continueSingleLine = false
			singleComment = false
			inComment = false
			return
		}
		if continueSingleLine && singleComment {
			prevComment := comments[len(comments)-1]
			if prevComment.Change == i.Change() {
				prevComment.String += "\n" + string(curCmt)
			}
			comments[len(comments)-1] = prevComment
		} else {
			cmt := comment{
				Line:   cstart,
				String: string(curCmt),
				Change: i.Change(),
			}
			comments = append(comments, cmt)
			continueSingleLine = false
		}
		ridx = 0
		if singleComment {
			continueSingleLine = true
		}
		singleComment = false
		inComment = false
	}
	for i.Scan() {
		line = i.Text()
		llen := len(line) - 1
		// Make sure we include blank lines
		if inComment && !singleComment && llen == -1 {
			curCmt = append(curCmt, '\n')
		}
		if singleComment && len(strings.TrimSpace(string(curCmt))) == 0 {
			continueSingleLine = false
			singleComment = false
			inComment = false
		}
		var escaped bool
		// XXX refactor parsing to "consume-to-end"
	Chars:
		for ci, c := range line {
			if escaped {
				escaped = false
				continue
			}
			if inComment {
				if !singleComment { // Check if we're ending
					if ridx >= len(end) {
						msg := fmt.Sprintf("ridx = %d, ci = %d, line = %s", ridx, ci, line)
						panic(msg)
					}
					if c == end[ridx] {
						if len(end) > ridx+1 {
							ridx++
						} else {
							addComment()
						}
						continue Chars
					} else {
						// Reset the next index
						ridx = 0
					}
				}
				curCmt = append(curCmt, c)
				// If single comment and we're at the end of the line, process it
				if singleComment && ci == llen {
					addComment()
				}

				// tack on the carriage return if we're at the end of the line
				if !singleComment && ci == llen {
					curCmt = append(curCmt, '\n')
				}
			} else if inString {
				if ridx >= len(syn.stringLits) {
					msg := fmt.Sprintf("ridx = %d, ci = %d, line = %#v", ridx, ci, line)
					panic(msg)
				}
				// Escape the next char
				if c == '\\' {
					escaped = true
					continue
				}
				if c == syn.stringLits[ridx] {
					// In a string... check to see if we're closing
					inString = false
					ridx = 0
					continue Chars
				}
			} else {
				for i, l := range syn.stringLits {
					if l == c {
						inString = true
						ridx = i
						continueSingleLine = false
						continue Chars
					}
				}
				for _, l := range syn.line {
					rl := []rune(l)
					if ridx >= len(rl) {
						continue // Try all options
					}
					if c == rl[ridx] {
						if len(rl) > ridx+1 {
							ridx++
						} else {
							ridx = 0
							inComment = true
							curCmt = []rune{}
							cstart = i.Line()
							singleComment = true
						}
						continue Chars
					}
				}
				for _, l := range syn.block {
					start := []rune(l[0])
					if ridx >= len(start) {
						continue // Try all options
					}
					if c == start[ridx] {
						if len(start) > ridx+1 {
							ridx++
						} else {
							ridx = 0
							inComment = true
							if ci == llen {
								curCmt = []rune{'\n'}
							} else {
								curCmt = []rune{}
							}
							cstart = i.Line()
							end = []rune(l[1])
						}
						continueSingleLine = false
						continue Chars
					}
				}
				// Otherwise, we're in uninteresting code.
				continueSingleLine = false
				if ridx > 0 { // we had gotten a false pos on a sequence of comment chars
					ridx = 0
				}
			}
		}
	}
	// Close out if comment is at EOF
	if inComment {
		addComment()
	}
	return comments
}
