package parse

import (
	"os"
	"strings"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"ser1.net/legume/reader"
)

func Test_parse(t *testing.T) {
	tests := []struct {
		name string
		lang string
		input string
		want []comment
	}{
		{"base", "go", "// TODO testing", []comment{comment{Line: 1, String: " TODO testing"}}},
		{"indent", "go", "\t// TODO testing", []comment{comment{Line: 1, String: " TODO testing"}}},
		{"inline", "c", "   a=1; // TODO testing", []comment{comment{Line: 1, String: " TODO testing"}}},
	}
	for _, tt := range tests {
		t.Run  (tt.name, func(t *testing.T) {
			l := Languages[tt.lang]
			r := reader.New(strings.NewReader(tt.input), tt.lang, tt.name)
			assert.Equal(t, tt.want, parse(&r, l))
		})
	}
}

func Test_fileTests(t *testing.T) {
	tests := []struct {
		name string
		lang string
		fname string
		want []comment
	}{
		{"java", "java", "test1.java", []comment{
			comment{ Line: 1, String: " 00000\n\n "},
			comment{ Line:   3, String: ` 123 ` },
			comment{ Line:   7, String: ` ABC DEF /*` },
			comment{ Line:   11, String: ` A1B2 ` },
		}},
		{"c", "c", "test2.c", []comment{
				comment{ Line:   4, String: " A comment" },
				comment{ Line: 8, String: "\n\t A longer comment\n\t Multiple lines\n\t "},
		}},
	}
	for _, tt := range tests {
		l := Languages[tt.lang]
		i, err := os.Open("../test/"+tt.fname)
		assert.NoError(t, err)
		t.Run  (tt.name, func(t *testing.T) {
			r := reader.New(i, tt.lang, tt.fname)
			assert.Equal(t, tt.want, parse(&r, l))
		})
	}
}

func Test_Parse(t *testing.T) {
	tests := []struct {
		name string
		lang string
		fname string
		wantN int
		wantL int
		want string
	}{
		{"kakoune", "c", "input_handler.cc", 0, 564, "proper handling of char/column" },
	}
	for _, tt := range tests {
		i, err := os.Open("../test/"+tt.fname)
		assert.NoError(t, err)
		t.Run  (tt.name, func(t *testing.T) {
			r := reader.New(i, tt.lang, tt.fname)
			is := Parse(&r)
			require.Greater(t, len(is), tt.wantN)
			assert.Equal(t, tt.wantL, is[tt.wantN].Row)
			assert.Equal(t, tt.want, is[tt.wantN].Description)
		})
	}
}
