package parse

var Languages map[string]syntax = map[string]syntax{
	"hs":      haskell,
	"lhs":     haskell,
	"hsc":     haskell,
	"chs":     haskell,
	"cabal":   haskell,
	"elm":     haskell,
	"go":      golang,
	"c":       c,
	"h":       c,
	"cc":      c,
	"hh":      c,
	"cpp":     c,
	"hpp":     c,
	"java":    c,
	"glsl":    c,
	"xrl":     c,
	"js":      javascript,
	"ts":      javascript,
	"pas":     pascal,
	"pp":      pascal,
	"inc":     pascal,
	"py":      python,
	"coffee":  python,
	"rb":      ruby,
	"pl":      perl,
	"pm":      perl,
	"t":       perl,
	"sh":      perl,
	"kak":     perl,
	"nix":     nix,
	"xml":     xml,
	"html":    xml,
	"erl":     erlang,
	"hrl":     erlang,
	"escript": erlang,
	"yrl":     erlang,
	"ml":      ocaml,
	"mli":     ocaml,
	"rs":      rust,
	"rlib":    rust,
	"sml":     sml,
	"todo":    todo,
	"yml":     yaml,
	//"rst": rst,
	//"txt": txt,
}

type syntax struct {
	line       []string   // { COMMENTSTART }
	block      [][]string // { {COMMSTART, COMMEND} }
	stringLits []rune     // { STRINGLITERAL }
}

var haskell syntax = syntax{[]string{"--"}, [][]string{{"{-", "-}"}}, []rune{'"', '\''}}
var c syntax = syntax{[]string{"//"}, [][]string{{"/*", "*/"}}, []rune{'"', '\''}}
var golang syntax = syntax{[]string{"//"}, [][]string{{"/*", "*/"}}, []rune{'"', '\'', '`'}}
var javascript syntax = syntax{[]string{"//"}, [][]string{{"/*", "*/"}}, []rune{'"', '\''}}
var pascal syntax = syntax{[]string{"//"}, [][]string{{"{", "}"}, {"(*", "*)"}}, []rune{'\''}}
var python syntax = syntax{[]string{"#"}, [][]string{{"\"\"\"", "\"\"\""}}, []rune{'"', '\''}}
var ruby syntax = syntax{[]string{"#"}, [][]string{{"=begin", "=end"}}, []rune{'"', '\''}}
var perl syntax = syntax{[]string{"#"}, [][]string{}, []rune{'"', '\''}}
var nix syntax = syntax{[]string{"#"}, [][]string{{"/*", "*/"}}, []rune{'"', '\''}}
var xml syntax = syntax{[]string{}, [][]string{{"<!--", "-->"}}, []rune{'"', '\''}}
var erlang syntax = syntax{[]string{"%"}, [][]string{}, []rune{'"'}}
var ocaml syntax = syntax{[]string{}, [][]string{{"(*", "*)"}}, []rune{'"', '\''}}
var rust syntax = syntax{[]string{"//!", "///", "//"}, [][]string{}, []rune{'"', '\''}}
var sml syntax = syntax{[]string{}, [][]string{{"(*", "*)"}}, []rune{'"'}}
var todo syntax = syntax{[]string{"#"}, [][]string{}, []rune{}}
var yaml syntax = syntax{[]string{"#"}, [][]string{}, []rune{}}

/*
var rst        parsyntax = source Rstsyntax
text       = (:[]) . MultiLine 1 <$> many anyChar
*/
