package xml

import (
	gxml "encoding/xml"
	"io"
	"time"

	. "ser1.net/legume"
)

func ToString(is []Issue, out io.Writer) {
	os := make(XMLIssues, len(is))
	for k, i := range is {
		os[k] = Convert(i)
	}
	bs, err := gxml.Marshal(os)
	if err != nil {
		panic(err)
	}
	out.Write(bs)
}

type XMLIssues []XMLIssue

func (s XMLIssues) MarshalXML(e *gxml.Encoder, start gxml.StartElement) error {
	t := gxml.StartElement{Name: gxml.Name{"", "issues"}}
	err := e.EncodeToken(t)
	if err != nil {
		return err
	}
	for _, i := range s {
		err := e.Encode(i)
		if err != nil {
			return err
		}
	}
	u := gxml.EndElement{t.Name}
	err = e.EncodeToken(u)
	if err != nil {
		return err
	}

	err = e.Flush()
	if err != nil {
		return err
	}

	return nil
}

type XMLIssue struct {
	XMLName     gxml.Name  `xml:"issue"`
	Path        string     `xml:"path,attr"`
	Row         int        `xml:"row,attr"`
	Description string     `xml:"description"`
	Tags        TagMap     `xml:"tags,omitempty"`
	Contexts    []string   `xml:"contexts"`
	Projects    []string   `xml:"projects"`
	Date        *time.Time `xml:"created,attr,omitempty"`
	Priority    string     `xml:"priority,attr"`
	Change      int        `xml:"changeType,attr"`
}

func Convert(i Issue) XMLIssue {
	return XMLIssue{
		Path:        i.Path,
		Row:         i.Row,
		Description: i.Description,
		Tags:        TagMap(i.Tags),
		Contexts:    i.Contexts,
		Projects:    i.Projects,
		Date:        i.Date,
		Priority:    i.Priority,
		Change:      i.Change,
	}
}

type TagMap map[string]string

func (s TagMap) MarshalXML(e *gxml.Encoder, start gxml.StartElement) error {
	tokens := []gxml.Token{start}
	for key, value := range s {
		t := gxml.StartElement{Name: gxml.Name{"", "tag"}}
		t.Attr = []gxml.Attr{gxml.Attr{gxml.Name{"", key}, value}}
		tokens = append(tokens, t, gxml.EndElement{t.Name})
	}
	tokens = append(tokens, gxml.EndElement{start.Name})

	for _, t := range tokens {
		err := e.EncodeToken(t)
		if err != nil {
			return err
		}
	}

	// flush to ensure tokens are written
	err := e.Flush()
	if err != nil {
		return err
	}

	return nil
}
